package com.xs.core.common.aspect;

import com.xs.core.common.annotation.OperationLog;
import com.xs.core.service.log.ISysAppLogService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.concurrent.CompletableFuture;

@Aspect
@Component
public class OperationLogAspect {
    @Autowired
    private ISysAppLogService sysLogService;

    @Pointcut("@annotation(com.xs.core.common.annotation.OperationLog)")
    public void logPointCut() {
    }

    @Around("logPointCut()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        long beginTime = System.currentTimeMillis();
        Object result = null;
        // 执行方法
        result = point.proceed();
        // 执行时长(毫秒)
        long time = System.currentTimeMillis() - beginTime;
        // 保存日志
        saveLog(point, time, result, null);
        return result;
    }

    private void saveLog(ProceedingJoinPoint joinPoint, long time, Object result, Exception e) {
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();

        // 获取注解信息
        OperationLog logAnnotation = method.getAnnotation(OperationLog.class);
        String description = null;
        String operationType = null;
        String module = null;
        String methodName = method.getDeclaringClass().getName() + "." + method.getName();
        if (logAnnotation != null) {
            description = logAnnotation.description();
            operationType = logAnnotation.operationType();
            module = logAnnotation.module();
        }
        sysLogService.saveAppLog(description, operationType, module, methodName, joinPoint.getArgs(), result, time, e);
    }

}
